MODULE diaptr
   !!======================================================================
   !!                       ***  MODULE  diaptr  ***
   !! Ocean physics:  Computes meridonal transports and zonal means
   !!=====================================================================
   !! History :  1.0  ! 2003-09  (C. Talandier, G. Madec)  Original code
   !!            2.0  ! 2006-01  (A. Biastoch)  Allow sub-basins computation
   !!            3.2  ! 2010-03  (O. Marti, S. Flavoni) Add fields
   !!            3.3  ! 2010-10  (G. Madec)  dynamical allocation
   !!----------------------------------------------------------------------

   !!----------------------------------------------------------------------
   !!   dia_ptr      : Poleward Transport Diagnostics module
   !!   dia_ptr_init : Initialization, namelist read
   !!   dia_ptr_wri  : Output of poleward fluxes
   !!   ptr_vjk      : "zonal" sum computation of a "meridional" flux array
   !!   ptr_tjk      : "zonal" mean computation of a tracer field
   !!   ptr_vj       : "zonal" and vertical sum computation of a "meridional" flux array
   !!                   (Generic interface to ptr_vj_3d, ptr_vj_2d)
   !!----------------------------------------------------------------------
   USE oce              ! ocean dynamics and active tracers
   USE dom_oce          ! ocean space and time domain
   USE phycst           ! physical constants
   USE ldftra_oce       ! ocean active tracers: lateral physics
   USE dianam           !
   USE diaptr_oce
   USE iom              ! IOM library
   USE ioipsl           ! IO-IPSL library
   USE in_out_manager   ! I/O manager
   USE lib_mpp          ! MPP library
   USE lbclnk           ! lateral boundary condition - processor exchanges
   USE timing           ! preformance summary
   USE wrk_nemo         ! working arrays

   IMPLICIT NONE
   PRIVATE

   PUBLIC   dia_ptr_init   ! call in opa module
   PUBLIC   dia_ptr        ! call in step module

   !! * Substitutions
#  include "domzgr_substitute.h90"
#  include "vectopt_loop_substitute.h90"
   !!----------------------------------------------------------------------
   !! NEMO/OPA 3.3 , NEMO Consortium (2010)
   !! $Id: diaptr.F90 4624 2014-04-28 12:09:03Z acc $ 
   !! Software governed by the CeCILL licence     (NEMOGCM/NEMO_CeCILL.txt)
   !!----------------------------------------------------------------------
CONTAINS


   SUBROUTINE dia_ptr( kt )
      !!----------------------------------------------------------------------
      !!                  ***  ROUTINE dia_ptr  ***
      !!----------------------------------------------------------------------
      USE oce,     vt  =>   ua   ! use ua as workspace
      USE oce,     vs  =>   va   ! use va as workspace
      IMPLICIT none
      !!
      INTEGER, INTENT(in) ::   kt   ! ocean time step index
      !
      INTEGER  ::   ji, jj, jk, jn   ! dummy loop indices
      REAL(wp) ::   zv               ! local scalar
      !!----------------------------------------------------------------------
      !
      IF( nn_timing == 1 )   CALL timing_start('dia_ptr')
      !
      IF( kt == nit000 .OR. MOD( kt, nn_fptr ) == 0 )   THEN
         !
         IF( MOD( kt, nn_fptr ) == 0 ) THEN 
            !
            IF( ln_diaznl ) THEN               ! i-mean temperature and salinity
               DO jn = 1, nptr
                  tn_jk(:,:,jn) = ptr_tjk( tsn(:,:,:,jp_tem), btmsk(:,:,jn) ) * r1_sjk(:,:,jn)
                  sn_jk(:,:,jn) = ptr_tjk( tsn(:,:,:,jp_sal), btmsk(:,:,jn) ) * r1_sjk(:,:,jn)
               END DO
            ENDIF
            !
            !                          ! horizontal integral and vertical dz 
            !                                ! eulerian velocity
            v_msf(:,:,1) = ptr_vjk( vn(:,:,:) ) 
            DO jn = 2, nptr
               v_msf(:,:,jn) = ptr_vjk( vn(:,:,:), btmsk(:,:,jn)*btm30(:,:) ) 
            END DO
#if defined key_diaeiv
            DO jn = 1, nptr                  ! bolus velocity
               v_msf_eiv(:,:,jn) = ptr_vjk( v_eiv(:,:,:), btmsk(:,:,jn) )   ! here no btm30 for MSFeiv
            END DO
            !                                ! add bolus stream-function to the eulerian one
            v_msf(:,:,:) = v_msf(:,:,:) + v_msf_eiv(:,:,:)
#endif
            !
            !                          ! Transports
            !                                ! local heat & salt transports at T-points  ( tsn*mj[vn+v_eiv] )
            vt(:,:,jpk) = 0._wp   ;   vs(:,:,jpk) = 0._wp
            DO jk= 1, jpkm1
               DO jj = 2, jpj
                  DO ji = 1, jpi
#if defined key_diaeiv 
                     zv = ( vn(ji,jj,jk) + vn(ji,jj-1,jk) + v_eiv(ji,jj,jk) + v_eiv(ji,jj-1,jk) ) * 0.5_wp
#else
                     zv = ( vn(ji,jj,jk) + vn(ji,jj-1,jk) ) * 0.5_wp
#endif 
                     vt(ji,jj,jk) = zv * tsn(ji,jj,jk,jp_tem)
                     vs(ji,jj,jk) = zv * tsn(ji,jj,jk,jp_sal)
                  END DO
               END DO
            END DO
!!gm useless as overlap areas are not used in ptr_vjk
            CALL lbc_lnk( vs, 'V', -1. )   ;   CALL lbc_lnk( vt, 'V', -1. )
!!gm
            !                                ! heat & salt advective transports (approximation)
            htr(:,1) = SUM( ptr_vjk( vt(:,:,:) ) , 2 ) * rc_pwatt   ! SUM over jk + conversion
            str(:,1) = SUM( ptr_vjk( vs(:,:,:) ) , 2 ) * rc_ggram
            DO jn = 2, nptr 
               htr(:,jn) = SUM( ptr_vjk( vt(:,:,:), btmsk(:,:,jn)*btm30(:,:) ) , 2 ) * rc_pwatt   ! mask Southern Ocean
               str(:,jn) = SUM( ptr_vjk( vs(:,:,:), btmsk(:,:,jn)*btm30(:,:) ) , 2 ) * rc_ggram   ! mask Southern Ocean
            END DO

            IF( ln_ptrcomp ) THEN            ! overturning transport
               htr_ove(:) = SUM( v_msf(:,:,1) * tn_jk(:,:,1), 2 ) * rc_pwatt   ! SUM over jk + conversion
               str_ove(:) = SUM( v_msf(:,:,1) * sn_jk(:,:,1), 2 ) * rc_ggram
            END IF
            !                                ! Advective and diffusive transport
            htr_adv(:) = htr_adv(:) * rc_pwatt        ! these are computed in tra_adv... and tra_ldf... routines 
            htr_ldf(:) = htr_ldf(:) * rc_pwatt        ! here just the conversion in PW and Gg
            str_adv(:) = str_adv(:) * rc_ggram
            str_ldf(:) = str_ldf(:) * rc_ggram

#if defined key_diaeiv
            DO jn = 1, nptr                  ! Bolus component
               htr_eiv(:,jn) = SUM( v_msf_eiv(:,:,jn) * tn_jk(:,:,jn), 2 ) * rc_pwatt   ! SUM over jk
               str_eiv(:,jn) = SUM( v_msf_eiv(:,:,jn) * sn_jk(:,:,jn), 2 ) * rc_ggram   ! SUM over jk
            END DO
#endif
            !                                ! "Meridional" Stream-Function
            DO jn = 1, nptr
               DO jk = 2, jpk 
                  v_msf    (:,jk,jn) = v_msf    (:,jk-1,jn) + v_msf    (:,jk,jn)       ! Eulerian j-Stream-Function
#if defined key_diaeiv
                  v_msf_eiv(:,jk,jn) = v_msf_eiv(:,jk-1,jn) + v_msf_eiv(:,jk,jn)       ! Bolus    j-Stream-Function

#endif
               END DO
            END DO
            v_msf    (:,:,:) = v_msf    (:,:,:) * rc_sv       ! converte in Sverdrups
#if defined key_diaeiv
            v_msf_eiv(:,:,:) = v_msf_eiv(:,:,:) * rc_sv
#endif
         ENDIF
         !
         CALL dia_ptr_wri( kt )                        ! outputs
         !
      ENDIF
      !
      IF( nn_timing == 1 )   CALL timing_stop('dia_ptr')
      !
   END SUBROUTINE dia_ptr


   SUBROUTINE dia_ptr_init
      !!----------------------------------------------------------------------
      !!                  ***  ROUTINE dia_ptr_init  ***
      !!                   
      !! ** Purpose :   Initialization, namelist read
      !!----------------------------------------------------------------------
      INTEGER ::   jn, ji, jj           ! dummy loop indices 
      INTEGER ::   inum, ierr   ! local integers
      INTEGER ::   ios          ! Local integer output status for namelist read
#if defined key_mpp_mpi
      INTEGER, DIMENSION(1) :: iglo, iloc, iabsf, iabsl, ihals, ihale, idid
#endif
      !!
      NAMELIST/namptr/ ln_diaptr, ln_diaznl, ln_subbas, ln_ptrcomp, nn_fptr, nn_fwri
      !!----------------------------------------------------------------------

      REWIND( numnam_ref )              ! Namelist namptr in reference namelist : Poleward transport
      READ  ( numnam_ref, namptr, IOSTAT = ios, ERR = 901)
901   IF( ios /= 0 ) CALL ctl_nam ( ios , 'namptr in reference namelist', lwp )

      REWIND( numnam_cfg )              ! Namelist namptr in configuration namelist : Poleward transport
      READ  ( numnam_cfg, namptr, IOSTAT = ios, ERR = 902 )
902   IF( ios /= 0 ) CALL ctl_nam ( ios , 'namptr in configuration namelist', lwp )
      IF(lwm) WRITE ( numond, namptr )

      IF(lwp) THEN                     ! Control print
         WRITE(numout,*)
         WRITE(numout,*) 'dia_ptr_init : poleward transport and msf initialization'
         WRITE(numout,*) '~~~~~~~~~~~~'
         WRITE(numout,*) '   Namelist namptr : set ptr parameters'
         WRITE(numout,*) '      Poleward heat & salt transport (T) or not (F)      ln_diaptr  = ', ln_diaptr
         WRITE(numout,*) '      Overturning heat & salt transport                  ln_ptrcomp = ', ln_ptrcomp
         WRITE(numout,*) '      T & S zonal mean and meridional stream function    ln_diaznl  = ', ln_diaznl 
         WRITE(numout,*) '      Global (F) or glo/Atl/Pac/Ind/Indo-Pac basins      ln_subbas  = ', ln_subbas
         WRITE(numout,*) '      Frequency of computation                           nn_fptr    = ', nn_fptr
         WRITE(numout,*) '      Frequency of outputs                               nn_fwri    = ', nn_fwri
      ENDIF
      
      IF( ln_diaptr) THEN  
     
         IF( nn_timing == 1 )   CALL timing_start('dia_ptr_init')
      
         IF( ln_subbas ) THEN   ;   nptr = 5       ! Global, Atlantic, Pacific, Indian, Indo-Pacific
         ELSE                   ;   nptr = 1       ! Global only
         ENDIF

         !                                      ! allocate dia_ptr arrays
         IF( dia_ptr_alloc() /= 0 )   CALL ctl_stop( 'STOP', 'dia_ptr_init : unable to allocate arrays' )

         rc_pwatt = rc_pwatt * rau0 * rcp          ! conversion from K.s-1 to PetaWatt

         IF( lk_mpp )   CALL mpp_ini_znl( numout )     ! Define MPI communicator for zonal sum

         IF( ln_subbas ) THEN                ! load sub-basin mask
            CALL iom_open( 'subbasins', inum )
            CALL iom_get( inum, jpdom_data, 'atlmsk', btmsk(:,:,2) )   ! Atlantic basin
            CALL iom_get( inum, jpdom_data, 'pacmsk', btmsk(:,:,3) )   ! Pacific  basin
            CALL iom_get( inum, jpdom_data, 'indmsk', btmsk(:,:,4) )   ! Indian   basin
            CALL iom_close( inum )
            btmsk(:,:,5) = MAX ( btmsk(:,:,3), btmsk(:,:,4) )          ! Indo-Pacific basin
            WHERE( gphit(:,:) < -30._wp)   ;   btm30(:,:) = 0._wp      ! mask out Southern Ocean
            ELSE WHERE                     ;   btm30(:,:) = tmask(:,:,1)
            END WHERE
         ENDIF
         btmsk(:,:,1) = tmask_i(:,:)                                   ! global ocean
      
         DO jn = 1, nptr
            btmsk(:,:,jn) = btmsk(:,:,jn) * tmask_i(:,:)               ! interior domain only
         END DO
      
         IF( lk_vvl )   CALL ctl_stop( 'diaptr: error in vvl case as constant i-mean surface is used' )

         !                                   ! i-sum of e1v*e3v surface and its inverse
         DO jn = 1, nptr
            sjk(:,:,jn) = ptr_tjk( tmask(:,:,:), btmsk(:,:,jn) )
            r1_sjk(:,:,jn) = 0._wp
            WHERE( sjk(:,:,jn) /= 0._wp )   r1_sjk(:,:,jn) = 1._wp / sjk(:,:,jn)
         END DO

      ! Initialise arrays to zero because diatpr is called before they are first calculated
      ! Note that this means diagnostics will not be exactly correct when model run is restarted.
      htr_adv(:) = 0._wp ; str_adv(:) =  0._wp ;  htr_ldf(:) = 0._wp ; str_ldf(:) =  0._wp

      ! Reference latitude (used in plots)
      ! ------------------
      !                                           ! =======================
      IF( cp_cfg == "orca" ) THEN                 !   ORCA configurations
        !                                        ! =======================
        IF( jp_cfg == 05  )   nxline = 192   ! i-line that passes near the North Pole
        IF( jp_cfg == 025 )   nxline = 384   ! i-line that passes near the North Pole
        IF( jp_cfg == 1   )   nxline =  96   ! i-line that passes near the North Pole
        IF( jp_cfg == 2   )   nxline =  48   ! i-line that passes near the North Pole
        IF( jp_cfg == 4   )   nxline =  24   ! i-line that passes near the North Pole
        !                                         !=======================
      ELSE                                        !   OTHER configurations 
        !                                        ! =======================
        nxline = 1
        !
      ENDIF

      IF( nn_timing == 1 )   CALL timing_stop('dia_ptr_init')
      !
      ENDIF 
      ! 
   END SUBROUTINE dia_ptr_init

#if defined key_iomput

   SUBROUTINE dia_ptr_wri( kt )
      !!---------------------------------------------------------------------
      !!                ***  ROUTINE dia_ptr_wri  ***
      !!
      !! ** Purpose :   output of poleward fluxes
      !!
      !! ** Method  :   NetCDF file
      !!----------------------------------------------------------------------
      !!
      INTEGER, INTENT(in) ::   kt   ! ocean time-step index
      !!
      REAL(wp), POINTER, DIMENSION(:,:    ) :: z2d 
      REAL(wp), POINTER, DIMENSION(:,:,:  ) :: z3d  
      REAL(wp), POINTER, DIMENSION(:,:,:,:) :: z4d 
      !!----------------------------------------------------------------------


      CALL wrk_alloc( jpi, jpj, jpk, nptr, z4d )
      CALL wrk_alloc( jpi, jpj,      nptr, z3d )
      CALL wrk_alloc( jpi, jpj,            z2d )
      !
      z4d(:,:,:,:) = 0._wp
      z3d(:,:,:  ) = 0._wp
      z2d(:,:    ) = 0._wp
      !
      IF( ln_diaznl ) THEN 
        !
        z4d(nxline,:,:,1) = sjk(:,:,1)
        CALL iom_put( 'zosrfglo', z4d(:,:,:,1) )
        z4d(nxline,:,:,1) = tn_jk(:,:,1)
        CALL iom_put( 'zotemglo', z4d(:,:,:,1) )
        z4d(nxline,:,:,1) = sn_jk(:,:,1)
        CALL iom_put( 'zosalglo', z4d(:,:,:,1) )
        ! overturning outputs:
        z4d(nxline,:,:,1) = v_msf(:,:,1)
        CALL iom_put( 'zomsfglo', z4d(:,:,:,1) )
#if defined key_diaeiv
        z4d(nxline,:,:,1) = v_msf_eiv(:,:,1)
        CALL iom_put( 'zomsfeiv', z4d(:,:,:,1) )
#endif
        IF (ln_subbas) THEN 
           !
           z4d(nxline,:,:,2) = sjk(:,:,2)
           z4d(nxline,:,:,3) = sjk(:,:,3)
           z4d(nxline,:,:,4) = sjk(:,:,4)
           z4d(nxline,:,:,5) = sjk(:,:,5)
           CALL iom_put( 'zosrfatl', z4d(:,:,:,2) )
           CALL iom_put( 'zosrfpac', z4d(:,:,:,3) )
           CALL iom_put( 'zosrfind', z4d(:,:,:,4) )
           CALL iom_put( 'zosrfipc', z4d(:,:,:,5) )
           !
           z4d(nxline,:,:,2) = tn_jk(:,:,2)
           z4d(nxline,:,:,3) = tn_jk(:,:,3)
           z4d(nxline,:,:,4) = tn_jk(:,:,4)
           z4d(nxline,:,:,5) = tn_jk(:,:,5)
           CALL iom_put( 'zotematl', z4d(:,:,:,2) )
           CALL iom_put( 'zotempac', z4d(:,:,:,3) )
           CALL iom_put( 'zotemind', z4d(:,:,:,4) )
           CALL iom_put( 'zotemipc', z4d(:,:,:,5) )
           !
           z4d(nxline,:,:,2) = sn_jk(:,:,2)
           z4d(nxline,:,:,3) = sn_jk(:,:,3)
           z4d(nxline,:,:,4) = sn_jk(:,:,4)
           z4d(nxline,:,:,5) = sn_jk(:,:,5)
           CALL iom_put( 'zosalatl', z4d(:,:,:,2) )
           CALL iom_put( 'zosalpac', z4d(:,:,:,3) )
           CALL iom_put( 'zosalind', z4d(:,:,:,4) )
           CALL iom_put( 'zosalipc', z4d(:,:,:,5) )
           !
           z4d(nxline,:,:,2) = v_msf(:,:,2)
           z4d(nxline,:,:,3) = v_msf(:,:,3)
           z4d(nxline,:,:,4) = v_msf(:,:,4)
           z4d(nxline,:,:,5) = v_msf(:,:,5)
           CALL iom_put( 'zomsfatl', z4d(:,:,:,2) )
           CALL iom_put( 'zomsfpac', z4d(:,:,:,3) )
           CALL iom_put( 'zomsfind', z4d(:,:,:,4) )
           CALL iom_put( 'zomsfipc', z4d(:,:,:,5) )
        END IF
     ENDIF

     ! heat transport outputs:
     IF( ln_subbas ) THEN
        !
        z3d(nxline,:,2) = htr(:,2)
        z3d(nxline,:,3) = htr(:,3)
        z3d(nxline,:,4) = htr(:,4)
        z3d(nxline,:,5) = htr(:,5)
        CALL iom_put( 'sohtatl', z3d(:,:,2) )
        CALL iom_put( 'sohtpac', z3d(:,:,3) )
        CALL iom_put( 'sohtind', z3d(:,:,4) )
        CALL iom_put( 'sohtipc', z3d(:,:,5) )
        !
        z3d(nxline,:,2) = str(:,2)
        z3d(nxline,:,3) = str(:,3)
        z3d(nxline,:,4) = str(:,4)
        z3d(nxline,:,5) = str(:,5)
        CALL iom_put( 'sostatl', z3d(:,:,2) )
        CALL iom_put( 'sostpac', z3d(:,:,3) )
        CALL iom_put( 'sostind', z3d(:,:,4) )
        CALL iom_put( 'sostipc', z3d(:,:,5) )
        !
     ENDIF
#if defined key_diaeiv
     z3d(nxline,:,1) = htr_eiv(:,1)
     CALL iom_put( 'sophteiv', z3d(:,:,1) )
     !
     z3d(nxline,:,1) = str_eiv(:,1)
     CALL iom_put( 'sopsteiv', z3d(:,:,1) )
#endif
     z2d(nxline,:) = htr_adv(:)
     CALL iom_put( 'sophtadv', z2d(:,:) )
     z2d(nxline,:) = htr_ldf(:)
     CALL iom_put( 'sophtldf', z2d(:,:) )
     !
     z2d(nxline,:) = str_adv(:)
     CALL iom_put( 'sopstadv', z2d(:,:) )
     z2d(nxline,:) = str_ldf(:)
     CALL iom_put( 'sopstldf', z2d(:,:) )
     !
     IF( ln_ptrcomp ) THEN 
        z2d(nxline,:) = htr_ove(:)
        CALL iom_put( 'sophtove', z2d(:,:) )
        !
        z2d(nxline,:) = str_ove(:)
        CALL iom_put( 'sopstove', z2d(:,:) )
     ENDIF
    !
    CALL wrk_dealloc( jpi, jpj, jpk, nptr, z4d )
    CALL wrk_dealloc( jpi, jpj,      nptr, z3d )
    CALL wrk_dealloc( jpi, jpj,            z2d )

   END SUBROUTINE dia_ptr_wri

#else

   SUBROUTINE dia_ptr_wri( kt )
      !!---------------------------------------------------------------------
      !!                ***  ROUTINE dia_ptr_wri  ***
      !!
      !! ** Purpose :   output of poleward fluxes
      !!
      !! ** Method  :   NetCDF file
      !!----------------------------------------------------------------------
      !!
      INTEGER, INTENT(in) ::   kt   ! ocean time-step index
      !!
      INTEGER, SAVE ::   nhoridz, ndepidzt, ndepidzw
      INTEGER, SAVE ::   ndim  , ndim_atl     , ndim_pac     , ndim_ind     , ndim_ipc
      INTEGER, SAVE ::           ndim_atl_30  , ndim_pac_30  , ndim_ind_30  , ndim_ipc_30
      INTEGER, SAVE ::   ndim_h, ndim_h_atl_30, ndim_h_pac_30, ndim_h_ind_30, ndim_h_ipc_30
      !!
      CHARACTER (len=40) ::   clhstnam, clop, clop_once, cl_comment   ! temporary names
      INTEGER            ::   it, itmod, ji, jj, jk            !
#if defined key_iomput
      INTEGER            ::   inum                                    ! temporary logical unit
#endif
      REAL(wp)           ::   zsto, zout, zdt, zjulian                ! temporary scalars
      !!
      REAL(wp), POINTER, DIMENSION(:)   ::   zphi, zfoo    ! 1D workspace
      REAL(wp), POINTER, DIMENSION(:,:) ::   z_1           ! 2D workspace
#if defined key_mpp_mpi
      INTEGER, DIMENSION(1) :: iglo, iloc, iabsf, iabsl, ihals, ihale, idid
#endif

      !!-------------------------------------------------------------------- 
      !
      CALL wrk_alloc( jpj      , zphi, zfoo )
      CALL wrk_alloc( jpj , jpk, z_1        )

      ! define time axis
      it    = kt / nn_fptr
      itmod = kt - nit000 + 1
      
      ! Initialization
      ! --------------
      IF( kt == nit000 ) THEN
         niter = ( nit000 - 1 ) / nn_fptr
         zdt = rdt
         IF( nacc == 1 )   zdt = rdtmin
         !
         IF(lwp) THEN
            WRITE(numout,*)
            WRITE(numout,*) 'dia_ptr_wri : poleward transport and msf writing: initialization , niter = ', niter
            WRITE(numout,*) '~~~~~~~~~~~~'
         ENDIF
         !
#if defined key_mpp_mpi 
         iglo (1) = jpjglo                   ! MPP case using MPI  ('key_mpp_mpi')
         iloc (1) = nlcj
         iabsf(1) = njmppt(narea)
         iabsl(:) = iabsf(:) + iloc(:) - 1
         ihals(1) = nldj - 1
         ihale(1) = nlcj - nlej
         idid (1) = 2
         CALL flio_dom_set( jpnj, nproc/jpni, idid, iglo, iloc, iabsf, iabsl, ihals, ihale, 'BOX', nidom_ptr )
#else
         nidom_ptr = FLIO_DOM_NONE
#endif

         ! Reference latitude (used in plots)
         ! ------------------
         !                                           ! =======================
         IF( cp_cfg == "orca" ) THEN                 !   ORCA configurations
           !                                        ! =======================
           zphi(1:jpj) = 0._wp
           DO ji = mi0(nxline), mi1(nxline) 
              zphi(1:jpj) = gphiv(ji,:)         ! if nxline is in the local domain
              ! Correct highest latitude for some configurations - will work if domain is parallelized in J ?
              IF( jp_cfg == 05 ) THEN
                 DO jj = mj0(jpjdta), mj1(jpjdta) 
                    zphi( jj ) = zphi(mj0(jpjdta-1)) + ( zphi(mj0(jpjdta-1))-zphi(mj0(jpjdta-2)) ) * 0.5_wp
                    zphi( jj ) = MIN( zphi(jj), 90._wp )
                 END DO
              END IF
              IF( jp_cfg == 1 .OR. jp_cfg == 2 .OR. jp_cfg == 4 ) THEN
                 DO jj = mj0(jpjdta-1), mj1(jpjdta-1) 
                    zphi( jj ) = 88.5_wp
                 END DO
                 DO jj = mj0(jpjdta  ), mj1(jpjdta  ) 
                    zphi( jj ) = 89.5_wp
                 END DO
              END IF
           END DO
           ! provide the correct zphi to all local domains
#if defined key_mpp_mpi
           CALL mpp_sum( zphi, jpj, ncomm_znl )        
#endif
           !                                        ! =======================
         ELSE                                       !   OTHER configurations 
           !                                        ! =======================
           zphi(1:jpj) = gphiv(1,:)             ! assume lat/lon coordinate, select the first i-line
           !
         ENDIF

         ! Work only on westmost processor (will not work if mppini2 is used)
#if defined key_mpp_mpi
         IF( l_znl_root ) THEN 
#endif
            !
            ! OPEN netcdf file 
            ! ----------------
            ! Define frequency of output and means
            zsto = nn_fptr * zdt
            IF( ln_mskland )   THEN    ! put 1.e+20 on land (very expensive!!)
               clop      = "ave(only(x))"
               clop_once = "once(only(x))"
            ELSE                       ! no use of the mask value (require less cpu time)
               clop      = "ave(x)"       
               clop_once = "once"
            ENDIF

            zout = nn_fwri * zdt
            zfoo(1:jpj) = 0._wp

            CALL ymds2ju( nyear, nmonth, nday, rdt, zjulian )  ! Compute julian date from starting date of the run
            zjulian = zjulian - adatrj                         ! set calendar origin to the beginning of the experiment

#if defined key_iomput
            ! Requested by IPSL people, use by their postpro...
            IF(lwp) THEN
               CALL dia_nam( clhstnam, nn_fwri,' ' )
               CALL ctl_opn( inum, 'date.file', 'REPLACE', 'FORMATTED', 'SEQUENTIAL', -1, numout, lwp, narea )
               WRITE(inum,*) clhstnam
               CLOSE(inum)
            ENDIF
#endif

            CALL dia_nam( clhstnam, nn_fwri, 'diaptr' )
            IF(lwp)WRITE( numout,*)" Name of diaptr NETCDF file : ", clhstnam

            ! Horizontal grid : zphi()
            CALL histbeg(clhstnam, 1, zfoo, jpj, zphi,   &
               1, 1, 1, jpj, niter, zjulian, zdt*nn_fptr, nhoridz, numptr, domain_id=nidom_ptr)
            ! Vertical grids : gdept_1d, gdepw_1d
            CALL histvert( numptr, "deptht", "Vertical T levels",   &
               &                   "m", jpk, gdept_1d, ndepidzt, "down" )
            CALL histvert( numptr, "depthw", "Vertical W levels",   &
               &                   "m", jpk, gdepw_1d, ndepidzw, "down" )
            !
            CALL wheneq ( jpj*jpk, MIN(sjk(:,:,1), 1._wp), 1, 1., ndex  , ndim  )      ! Lat-Depth
            CALL wheneq ( jpj    , MIN(sjk(:,1,1), 1._wp), 1, 1., ndex_h, ndim_h )     ! Lat

            IF( ln_subbas ) THEN
               z_1(:,1) = 1._wp
               WHERE ( gphit(jpi/2,:) < -30._wp )   z_1(:,1) = 0._wp
               DO jk = 2, jpk
                  z_1(:,jk) = z_1(:,1)
               END DO
               !                       ! Atlantic (jn=2)
               CALL wheneq ( jpj*jpk, MIN(sjk(:,:,2)         , 1._wp), 1, 1., ndex_atl     , ndim_atl      ) ! Lat-Depth
               CALL wheneq ( jpj*jpk, MIN(sjk(:,:,2)*z_1(:,:), 1._wp), 1, 1., ndex_atl_30  , ndim_atl_30   ) ! Lat-Depth
               CALL wheneq ( jpj    , MIN(sjk(:,1,2)*z_1(:,1), 1._wp), 1, 1., ndex_h_atl_30, ndim_h_atl_30 ) ! Lat
               !                       ! Pacific (jn=3)
               CALL wheneq ( jpj*jpk, MIN(sjk(:,:,3)         , 1._wp), 1, 1., ndex_pac     , ndim_pac      ) ! Lat-Depth
               CALL wheneq ( jpj*jpk, MIN(sjk(:,:,3)*z_1(:,:), 1._wp), 1, 1., ndex_pac_30  , ndim_pac_30   ) ! Lat-Depth
               CALL wheneq ( jpj    , MIN(sjk(:,1,3)*z_1(:,1), 1._wp), 1, 1., ndex_h_pac_30, ndim_h_pac_30 ) ! Lat
               !                       ! Indian (jn=4)
               CALL wheneq ( jpj*jpk, MIN(sjk(:,:,4)         , 1._wp), 1, 1., ndex_ind     , ndim_ind      ) ! Lat-Depth
               CALL wheneq ( jpj*jpk, MIN(sjk(:,:,4)*z_1(:,:), 1._wp), 1, 1., ndex_ind_30  , ndim_ind_30   ) ! Lat-Depth
               CALL wheneq ( jpj    , MIN(sjk(:,1,4)*z_1(:,1), 1._wp), 1, 1., ndex_h_ind_30, ndim_h_ind_30 ) ! Lat
               !                       ! Indo-Pacific (jn=5)
               CALL wheneq ( jpj*jpk, MIN(sjk(:,:,5)         , 1._wp), 1, 1., ndex_ipc     , ndim_ipc      ) ! Lat-Depth
               CALL wheneq ( jpj*jpk, MIN(sjk(:,:,5)*z_1(:,:), 1._wp), 1, 1., ndex_ipc_30  , ndim_ipc_30   ) ! Lat-Depth
               CALL wheneq ( jpj    , MIN(sjk(:,1,5)*z_1(:,1), 1._wp), 1, 1., ndex_h_ipc_30, ndim_h_ipc_30 ) ! Lat
            ENDIF
            ! 
#if defined key_diaeiv
            cl_comment = ' (Bolus part included)'
#else
            cl_comment = '                      '
#endif
            IF( ln_diaznl ) THEN             !  Zonal mean T and S
               CALL histdef( numptr, "zotemglo", "Zonal Mean Temperature","C" ,   &
                  1, jpj, nhoridz, jpk, 1, jpk, ndepidzt, 32, clop, zsto, zout )
               CALL histdef( numptr, "zosalglo", "Zonal Mean Salinity","PSU"  ,   &
                  1, jpj, nhoridz, jpk, 1, jpk, ndepidzt, 32, clop, zsto, zout )

               CALL histdef( numptr, "zosrfglo", "Zonal Mean Surface","m^2"   ,   &
                  1, jpj, nhoridz, jpk, 1, jpk, ndepidzt, 32, clop_once, zsto, zout )
               !
               IF (ln_subbas) THEN 
                  CALL histdef( numptr, "zotematl", "Zonal Mean Temperature: Atlantic","C" ,   &
                     1, jpj, nhoridz, jpk, 1, jpk, ndepidzt, 32, clop, zsto, zout )
                  CALL histdef( numptr, "zosalatl", "Zonal Mean Salinity: Atlantic","PSU"  ,   &
                     1, jpj, nhoridz, jpk, 1, jpk, ndepidzt, 32, clop, zsto, zout )
                  CALL histdef( numptr, "zosrfatl", "Zonal Mean Surface: Atlantic","m^2"   ,   &
                     1, jpj, nhoridz, jpk, 1, jpk, ndepidzt, 32, clop_once, zsto, zout )

                  CALL histdef( numptr, "zotempac", "Zonal Mean Temperature: Pacific","C"  ,   &
                     1, jpj, nhoridz, jpk, 1, jpk, ndepidzt, 32, clop, zsto, zout )
                  CALL histdef( numptr, "zosalpac", "Zonal Mean Salinity: Pacific","PSU"   ,   &
                     1, jpj, nhoridz, jpk, 1, jpk, ndepidzt, 32, clop, zsto, zout )
                  CALL histdef( numptr, "zosrfpac", "Zonal Mean Surface: Pacific","m^2"    ,   &
                     1, jpj, nhoridz, jpk, 1, jpk, ndepidzt, 32, clop_once, zsto, zout )

                  CALL histdef( numptr, "zotemind", "Zonal Mean Temperature: Indian","C"   ,   &
                     1, jpj, nhoridz, jpk, 1, jpk, ndepidzt, 32, clop, zsto, zout )
                  CALL histdef( numptr, "zosalind", "Zonal Mean Salinity: Indian","PSU"    ,   &
                     1, jpj, nhoridz, jpk, 1, jpk, ndepidzt, 32, clop, zsto, zout )
                  CALL histdef( numptr, "zosrfind", "Zonal Mean Surface: Indian","m^2"     ,   &
                     1, jpj, nhoridz, jpk, 1, jpk, ndepidzt, 32, clop_once, zsto, zout )

                  CALL histdef( numptr, "zotemipc", "Zonal Mean Temperature: Pacific+Indian","C" ,   &
                     1, jpj, nhoridz, jpk, 1, jpk, ndepidzt, 32, clop, zsto, zout )
                  CALL histdef( numptr, "zosalipc", "Zonal Mean Salinity: Pacific+Indian","PSU"  ,   &
                     1, jpj, nhoridz, jpk, 1, jpk, ndepidzt, 32, clop, zsto, zout )
                  CALL histdef( numptr, "zosrfipc", "Zonal Mean Surface: Pacific+Indian","m^2"   ,   &
                     1, jpj, nhoridz, jpk, 1, jpk, ndepidzt, 32, clop_once, zsto, zout )
               ENDIF
            ENDIF
            !
            !  Meridional Stream-Function (Eulerian and Bolus)
            CALL histdef( numptr, "zomsfglo", "Meridional Stream-Function: Global"//TRIM(cl_comment),"Sv" ,   &
               1, jpj, nhoridz, jpk, 1, jpk, ndepidzw, 32, clop, zsto, zout )
            IF( ln_subbas .AND. ln_diaznl ) THEN
               CALL histdef( numptr, "zomsfatl", "Meridional Stream-Function: Atlantic"//TRIM(cl_comment),"Sv" ,   &
                  1, jpj, nhoridz, jpk, 1, jpk, ndepidzw, 32, clop, zsto, zout )
               CALL histdef( numptr, "zomsfpac", "Meridional Stream-Function: Pacific"//TRIM(cl_comment),"Sv"  ,   &
                  1, jpj, nhoridz, jpk, 1, jpk, ndepidzw, 32, clop, zsto, zout )
               CALL histdef( numptr, "zomsfind", "Meridional Stream-Function: Indian"//TRIM(cl_comment),"Sv"   ,   &
                  1, jpj, nhoridz, jpk, 1, jpk, ndepidzw, 32, clop, zsto, zout )
               CALL histdef( numptr, "zomsfipc", "Meridional Stream-Function: Indo-Pacific"//TRIM(cl_comment),"Sv" ,&
                  1, jpj, nhoridz, jpk, 1, jpk, ndepidzw, 32, clop, zsto, zout )
            ENDIF
            !
            !  Heat transport 
            CALL histdef( numptr, "sophtadv", "Advective Heat Transport"      ,   &
               "PW", 1, jpj, nhoridz, 1, 1, 1, -99, 32, clop, zsto, zout )
            CALL histdef( numptr, "sophtldf", "Diffusive Heat Transport"      ,   &
               "PW",1, jpj, nhoridz, 1, 1, 1, -99, 32, clop, zsto, zout )
            IF ( ln_ptrcomp ) THEN 
               CALL histdef( numptr, "sophtove", "Overturning Heat Transport"    ,   &
                  "PW",1, jpj, nhoridz, 1, 1, 1, -99, 32, clop, zsto, zout )
            END IF
            IF( ln_subbas ) THEN
               CALL histdef( numptr, "sohtatl", "Heat Transport Atlantic"//TRIM(cl_comment),  &
                  "PW", 1, jpj, nhoridz, 1, 1, 1, -99, 32, clop, zsto, zout )
               CALL histdef( numptr, "sohtpac", "Heat Transport Pacific"//TRIM(cl_comment) ,  &
                  "PW", 1, jpj, nhoridz, 1, 1, 1, -99, 32, clop, zsto, zout )
               CALL histdef( numptr, "sohtind", "Heat Transport Indian"//TRIM(cl_comment)  ,  &
                  "PW", 1, jpj, nhoridz, 1, 1, 1, -99, 32, clop, zsto, zout )
               CALL histdef( numptr, "sohtipc", "Heat Transport Pacific+Indian"//TRIM(cl_comment), &
                  "PW", 1, jpj, nhoridz, 1, 1, 1, -99, 32, clop, zsto, zout )
            ENDIF
            !
            !  Salt transport 
            CALL histdef( numptr, "sopstadv", "Advective Salt Transport"      ,   &
               "Giga g/s", 1, jpj, nhoridz, 1, 1, 1, -99, 32, clop, zsto, zout )
            CALL histdef( numptr, "sopstldf", "Diffusive Salt Transport"      ,   &
               "Giga g/s", 1, jpj, nhoridz, 1, 1, 1, -99, 32, clop, zsto, zout )
            IF ( ln_ptrcomp ) THEN 
               CALL histdef( numptr, "sopstove", "Overturning Salt Transport"    ,   &
                  "Giga g/s", 1, jpj, nhoridz, 1, 1, 1, -99, 32, clop, zsto, zout )
            END IF
#if defined key_diaeiv
            ! Eddy induced velocity
            CALL histdef( numptr, "zomsfeiv", "Bolus Meridional Stream-Function: global",   &
               "Sv"      , 1, jpj, nhoridz, jpk, 1, jpk, ndepidzw, 32, clop, zsto, zout )
            CALL histdef( numptr, "sophteiv", "Bolus Advective Heat Transport",   &
               "PW"      , 1, jpj, nhoridz, 1, 1, 1, -99, 32, clop, zsto, zout )
            CALL histdef( numptr, "sopsteiv", "Bolus Advective Salt Transport",   &
               "Giga g/s", 1, jpj, nhoridz, 1, 1, 1, -99, 32, clop, zsto, zout )
#endif
            IF( ln_subbas ) THEN
               CALL histdef( numptr, "sostatl", "Salt Transport Atlantic"//TRIM(cl_comment)      ,  &
                  "Giga g/s", 1, jpj, nhoridz, 1, 1, 1, -99, 32, clop, zsto, zout )
               CALL histdef( numptr, "sostpac", "Salt Transport Pacific"//TRIM(cl_comment)      ,   &
                  "Giga g/s", 1, jpj, nhoridz, 1, 1, 1, -99, 32, clop, zsto, zout )
               CALL histdef( numptr, "sostind", "Salt Transport Indian"//TRIM(cl_comment)      ,    &
                  "Giga g/s", 1, jpj, nhoridz, 1, 1, 1, -99, 32, clop, zsto, zout )
               CALL histdef( numptr, "sostipc", "Salt Transport Pacific+Indian"//TRIM(cl_comment),  &
                  "Giga g/s", 1, jpj, nhoridz, 1, 1, 1, -99, 32, clop, zsto, zout )
            ENDIF
            !
            CALL histend( numptr )
            !
#if defined key_mpp_mpi
         END IF
#endif
      END IF

#if defined key_mpp_mpi
      IF( MOD( itmod, nn_fptr ) == 0 .AND. l_znl_root ) THEN
#else
      IF( MOD( itmod, nn_fptr ) == 0  ) THEN
#endif
         niter = niter + 1

         IF( ln_diaznl ) THEN 
            CALL histwrite( numptr, "zosrfglo", niter, sjk  (:,:,1) , ndim, ndex )
            CALL histwrite( numptr, "zotemglo", niter, tn_jk(:,:,1)  , ndim, ndex )
            CALL histwrite( numptr, "zosalglo", niter, sn_jk(:,:,1)  , ndim, ndex )

            IF (ln_subbas) THEN 
               CALL histwrite( numptr, "zosrfatl", niter, sjk(:,:,2), ndim_atl, ndex_atl )
               CALL histwrite( numptr, "zosrfpac", niter, sjk(:,:,3), ndim_pac, ndex_pac )
               CALL histwrite( numptr, "zosrfind", niter, sjk(:,:,4), ndim_ind, ndex_ind )
               CALL histwrite( numptr, "zosrfipc", niter, sjk(:,:,5), ndim_ipc, ndex_ipc )

               CALL histwrite( numptr, "zotematl", niter, tn_jk(:,:,2)  , ndim_atl, ndex_atl )
               CALL histwrite( numptr, "zosalatl", niter, sn_jk(:,:,2)  , ndim_atl, ndex_atl )
               CALL histwrite( numptr, "zotempac", niter, tn_jk(:,:,3)  , ndim_pac, ndex_pac )
               CALL histwrite( numptr, "zosalpac", niter, sn_jk(:,:,3)  , ndim_pac, ndex_pac )
               CALL histwrite( numptr, "zotemind", niter, tn_jk(:,:,4)  , ndim_ind, ndex_ind )
               CALL histwrite( numptr, "zosalind", niter, sn_jk(:,:,4)  , ndim_ind, ndex_ind )
               CALL histwrite( numptr, "zotemipc", niter, tn_jk(:,:,5)  , ndim_ipc, ndex_ipc )
               CALL histwrite( numptr, "zosalipc", niter, sn_jk(:,:,5)  , ndim_ipc, ndex_ipc )
            END IF
         ENDIF

         ! overturning outputs:
         CALL histwrite( numptr, "zomsfglo", niter, v_msf(:,:,1), ndim, ndex )
         IF( ln_subbas .AND. ln_diaznl ) THEN
            CALL histwrite( numptr, "zomsfatl", niter, v_msf(:,:,2) , ndim_atl_30, ndex_atl_30 )
            CALL histwrite( numptr, "zomsfpac", niter, v_msf(:,:,3) , ndim_pac_30, ndex_pac_30 )
            CALL histwrite( numptr, "zomsfind", niter, v_msf(:,:,4) , ndim_ind_30, ndex_ind_30 )
            CALL histwrite( numptr, "zomsfipc", niter, v_msf(:,:,5) , ndim_ipc_30, ndex_ipc_30 )
         ENDIF
#if defined key_diaeiv
         CALL histwrite( numptr, "zomsfeiv", niter, v_msf_eiv(:,:,1), ndim  , ndex   )
#endif

         ! heat transport outputs:
         IF( ln_subbas ) THEN
            CALL histwrite( numptr, "sohtatl", niter, htr(:,2)  , ndim_h_atl_30, ndex_h_atl_30 )
            CALL histwrite( numptr, "sohtpac", niter, htr(:,3)  , ndim_h_pac_30, ndex_h_pac_30 )
            CALL histwrite( numptr, "sohtind", niter, htr(:,4)  , ndim_h_ind_30, ndex_h_ind_30 )
            CALL histwrite( numptr, "sohtipc", niter, htr(:,5)  , ndim_h_ipc_30, ndex_h_ipc_30 )
            CALL histwrite( numptr, "sostatl", niter, str(:,2)  , ndim_h_atl_30, ndex_h_atl_30 )
            CALL histwrite( numptr, "sostpac", niter, str(:,3)  , ndim_h_pac_30, ndex_h_pac_30 )
            CALL histwrite( numptr, "sostind", niter, str(:,4)  , ndim_h_ind_30, ndex_h_ind_30 )
            CALL histwrite( numptr, "sostipc", niter, str(:,5)  , ndim_h_ipc_30, ndex_h_ipc_30 )
         ENDIF

         CALL histwrite( numptr, "sophtadv", niter, htr_adv     , ndim_h, ndex_h )
         CALL histwrite( numptr, "sophtldf", niter, htr_ldf     , ndim_h, ndex_h )
         CALL histwrite( numptr, "sopstadv", niter, str_adv     , ndim_h, ndex_h )
         CALL histwrite( numptr, "sopstldf", niter, str_ldf     , ndim_h, ndex_h )
         IF( ln_ptrcomp ) THEN 
            CALL histwrite( numptr, "sopstove", niter, str_ove(:) , ndim_h, ndex_h )
            CALL histwrite( numptr, "sophtove", niter, htr_ove(:) , ndim_h, ndex_h )
         ENDIF
#if defined key_diaeiv
         CALL histwrite( numptr, "sophteiv", niter, htr_eiv(:,1)  , ndim_h, ndex_h )
         CALL histwrite( numptr, "sopsteiv", niter, str_eiv(:,1)  , ndim_h, ndex_h )
#endif
         !
      ENDIF

      ! Close all files
#if defined key_mpp_mpi
      IF( kt == nitend .AND. l_znl_root )   CALL histclo( numptr )      ! Close the file
#else
      IF( kt == nitend )                    CALL histclo( numptr )      ! Close the file
#endif
      !
      CALL wrk_dealloc( jpj      , zphi, zfoo )
      CALL wrk_dealloc( jpj , jpk, z_1        )
      !
  END SUBROUTINE dia_ptr_wri

#endif

   !!======================================================================
END MODULE diaptr
