MODULE diaptr_oce
   !!======================================================================
   !!                       ***  MODULE  diaptr  ***
   !! Ocean physics:  Computes meridonal transports and zonal means
   !!=====================================================================
   !! History :  1.0  ! 2003-09  (C. Talandier, G. Madec)  Original code
   !!            2.0  ! 2006-01  (A. Biastoch)  Allow sub-basins computation
   !!            3.2  ! 2010-03  (O. Marti, S. Flavoni) Add fields
   !!            3.3  ! 2010-10  (G. Madec)  dynamical allocation
   !!----------------------------------------------------------------------

   USE par_oce
   USE dom_oce
   USE in_out_manager
   USE lib_mpp
   USE wrk_nemo

   IMPLICIT NONE
   PRIVATE

   INTERFACE ptr_vj
      MODULE PROCEDURE ptr_vj_3d, ptr_vj_2d
   END INTERFACE

   PUBLIC   ptr_vj         ! call by tra_ldf & tra_adv routines
   PUBLIC   ptr_vjk        ! call by tra_ldf & tra_adv routines
   PUBLIC   ptr_tjk        ! 
   PUBLIC   dia_ptr_alloc   ! call in opa module
   !                                  !!** namelist  namptr  **
   LOGICAL , PUBLIC ::   ln_diaptr     !: Poleward transport flag (T) or not (F)
   LOGICAL , PUBLIC ::   ln_subbas     !: Atlantic/Pacific/Indian basins calculation
   LOGICAL , PUBLIC ::   ln_diaznl     !: Add zonal means and meridional stream functions
   LOGICAL , PUBLIC ::   ln_ptrcomp    !: Add decomposition : overturning (and gyre, soon ...)
   INTEGER , PUBLIC ::   nn_fptr       !: frequency of ptr computation  [time step]
   INTEGER , PUBLIC ::   nn_fwri       !: frequency of ptr outputs      [time step]

   REAL(wp), ALLOCATABLE, SAVE, PUBLIC, DIMENSION(:) ::   htr_adv, htr_ldf, htr_ove   !: Heat TRansports (adv, diff, overturn.)
   REAL(wp), ALLOCATABLE, SAVE, PUBLIC, DIMENSION(:) ::   str_adv, str_ldf, str_ove   !: Salt TRansports (adv, diff, overturn.)
   
   REAL(wp), ALLOCATABLE, SAVE, PUBLIC, DIMENSION(:,:,:) ::   btmsk                  ! T-point basin interior masks
   REAL(wp), ALLOCATABLE, SAVE, PUBLIC, DIMENSION(:,:)   ::   btm30                  ! mask out Southern Ocean (=0 south of 30S)
   REAL(wp), ALLOCATABLE, SAVE, PUBLIC, DIMENSION(:,:)   ::   htr  , str             ! adv heat and salt transports (approx)
   REAL(wp), ALLOCATABLE, SAVE, PUBLIC, DIMENSION(:,:,:) ::   tn_jk, sn_jk , v_msf   ! i-mean T and S, j-Stream-Function
   REAL(wp), ALLOCATABLE, SAVE, PUBLIC, DIMENSION(:,:,:) ::   sjk  , r1_sjk          ! i-mean i-k-surface and its inverse        
   REAL(wp), ALLOCATABLE, SAVE, PUBLIC, DIMENSION(:,:)   ::   htr_eiv, str_eiv       ! bolus adv heat ans salt transports ('key_diaeiv')
   REAL(wp), ALLOCATABLE, SAVE, PUBLIC, DIMENSION(:,:,:) ::   v_msf_eiv              ! bolus j-streamfuction              ('key_diaeiv')


   INTEGER, SAVE, PUBLIC :: nxline       !
   INTEGER, PUBLIC ::   niter       !
   INTEGER, PUBLIC ::   nidom_ptr   !
   INTEGER, PUBLIC ::   numptr      ! logical unit for Poleward TRansports
   INTEGER, PUBLIC ::   nptr        ! = 1 (ln_subbas=F) or = 5 (glo, atl, pac, ind, ipc) (ln_subbas=T) 

   REAL(wp), PUBLIC ::   rc_sv    = 1.e-6_wp   ! conversion from m3/s to Sverdrup
   REAL(wp), PUBLIC ::   rc_pwatt = 1.e-15_wp  ! conversion from W    to PW (further x rau0 x Cp)
   REAL(wp), PUBLIC ::   rc_ggram = 1.e-6_wp   ! conversion from g    to Pg

   REAL(wp), PUBLIC, TARGET, DIMENSION(:)  , ALLOCATABLE, SAVE :: p_fval1d
   REAL(wp), PUBLIC, TARGET, DIMENSION(:,:), ALLOCATABLE, SAVE :: p_fval2d

   !! Integer, 1D workspace arrays. Not common enough to be implemented in 
   !! wrk_nemo module.
   INTEGER, ALLOCATABLE, SAVE, PUBLIC, DIMENSION(:) :: ndex  , ndex_atl     , ndex_pac     , ndex_ind     , ndex_ipc
   INTEGER, ALLOCATABLE, SAVE, PUBLIC, DIMENSION(:) ::         ndex_atl_30  , ndex_pac_30  , ndex_ind_30  , ndex_ipc_30
   INTEGER, ALLOCATABLE, SAVE, PUBLIC, DIMENSION(:) :: ndex_h, ndex_h_atl_30, ndex_h_pac_30, ndex_h_ind_30, ndex_h_ipc_30

   !! * Substitutions
#  include "domzgr_substitute.h90"
#  include "vectopt_loop_substitute.h90"

CONTAINS

   FUNCTION dia_ptr_alloc()
      !!----------------------------------------------------------------------
      !!                    ***  ROUTINE dia_ptr_alloc  ***
      !!----------------------------------------------------------------------
      INTEGER               ::   dia_ptr_alloc   ! return value
      INTEGER, DIMENSION(6) ::   ierr
      !!----------------------------------------------------------------------
      ierr(:) = 0
      !
      ALLOCATE( btmsk(jpi,jpj,nptr) ,           &
         &      htr_adv(jpj) , str_adv(jpj) ,   &
         &      htr_ldf(jpj) , str_ldf(jpj) ,   &
         &      htr_ove(jpj) , str_ove(jpj),    &
         &      htr(jpj,nptr) , str(jpj,nptr) , &
         &      tn_jk(jpj,jpk,nptr) , sn_jk (jpj,jpk,nptr) , v_msf(jpj,jpk,nptr) , &
         &      sjk  (jpj,jpk,nptr) , r1_sjk(jpj,jpk,nptr) , STAT=ierr(1)  )
         !
#if defined key_diaeiv
      ALLOCATE( htr_eiv(jpj,nptr) , str_eiv(jpj,nptr) , &
         &      v_msf_eiv(jpj,jpk,nptr) , STAT=ierr(2) )
#endif
      ALLOCATE( p_fval1d(jpj), p_fval2d(jpj,jpk), Stat=ierr(3))
      !
      ALLOCATE( btm30(jpi,jpj), STAT=ierr(4)  )

#if ! defined key_iomput
      ALLOCATE(ndex(jpj*jpk),        ndex_atl(jpj*jpk), ndex_pac(jpj*jpk), &
         &     ndex_ind(jpj*jpk),    ndex_ipc(jpj*jpk),                    &
         &     ndex_atl_30(jpj*jpk), ndex_pac_30(jpj*jpk), Stat=ierr(5))

      ALLOCATE(ndex_ind_30(jpj*jpk), ndex_ipc_30(jpj*jpk),                   &
         &     ndex_h(jpj),          ndex_h_atl_30(jpj), ndex_h_pac_30(jpj), &
         &     ndex_h_ind_30(jpj),   ndex_h_ipc_30(jpj), Stat=ierr(6) )
         !
#endif
         !
      dia_ptr_alloc = MAXVAL( ierr )
      IF(lk_mpp)   CALL mpp_sum( dia_ptr_alloc )
      !
   END FUNCTION dia_ptr_alloc


   FUNCTION ptr_vj_3d( pva )   RESULT ( p_fval )
      !!----------------------------------------------------------------------
      !!                    ***  ROUTINE ptr_vj_3d  ***
      !!
      !! ** Purpose :   i-k sum computation of a j-flux array
      !!
      !! ** Method  : - i-k sum of pva using the interior 2D vmask (vmask_i).
      !!              pva is supposed to be a masked flux (i.e. * vmask*e1v*e3v)
      !!
      !! ** Action  : - p_fval: i-k-mean poleward flux of pva
      !!----------------------------------------------------------------------
      REAL(wp) , INTENT(in), DIMENSION(jpi,jpj,jpk) ::   pva   ! mask flux array at V-point
      !!
      INTEGER                  ::   ji, jj, jk   ! dummy loop arguments
      INTEGER                  ::   ijpj         ! ???
      REAL(wp), POINTER, DIMENSION(:) :: p_fval  ! function value
      !!--------------------------------------------------------------------
      !
      p_fval => p_fval1d

      ijpj = jpj
      p_fval(:) = 0._wp
      DO jk = 1, jpkm1
         DO jj = 2, jpjm1
            DO ji = fs_2, fs_jpim1   ! Vector opt.
               p_fval(jj) = p_fval(jj) + pva(ji,jj,jk) * tmask_i(ji,jj) 
            END DO
         END DO
      END DO
#if defined key_mpp_mpi
      IF(lk_mpp)   CALL mpp_sum( p_fval, ijpj, ncomm_znl)
#endif
      !
   END FUNCTION ptr_vj_3d


   FUNCTION ptr_vj_2d( pva )   RESULT ( p_fval )
      !!----------------------------------------------------------------------
      !!                    ***  ROUTINE ptr_vj_2d  ***
      !!
      !! ** Purpose :   "zonal" and vertical sum computation of a i-flux array
      !!
      !! ** Method  : - i-k sum of pva using the interior 2D vmask (vmask_i).
      !!      pva is supposed to be a masked flux (i.e. * vmask*e1v*e3v)
      !!
      !! ** Action  : - p_fval: i-k-mean poleward flux of pva
      !!----------------------------------------------------------------------
      IMPLICIT none
      REAL(wp) , INTENT(in), DIMENSION(jpi,jpj) ::   pva   ! mask flux array at V-point
      !!
      INTEGER                  ::   ji,jj       ! dummy loop arguments
      INTEGER                  ::   ijpj        ! ???
      REAL(wp), POINTER, DIMENSION(:) :: p_fval ! function value
      !!--------------------------------------------------------------------
      ! 
      p_fval => p_fval1d

      ijpj = jpj
      p_fval(:) = 0._wp
      DO jj = 2, jpjm1
         DO ji = nldi, nlei   ! No vector optimisation here. Better use a mask ?
            p_fval(jj) = p_fval(jj) + pva(ji,jj) * tmask_i(ji,jj)
         END DO
      END DO
#if defined key_mpp_mpi
      CALL mpp_sum( p_fval, ijpj, ncomm_znl )
#endif
      ! 
   END FUNCTION ptr_vj_2d


   FUNCTION ptr_vjk( pva, pmsk )   RESULT ( p_fval )
      !!----------------------------------------------------------------------
      !!                    ***  ROUTINE ptr_vjk  ***
      !!
      !! ** Purpose :   i-sum computation of a j-velocity array
      !!
      !! ** Method  : - i-sum of pva using the interior 2D vmask (vmask_i).
      !!              pva is supposed to be a masked flux (i.e. * vmask)
      !!
      !! ** Action  : - p_fval: i-mean poleward flux of pva
      !!----------------------------------------------------------------------
      !!
      IMPLICIT none
      REAL(wp) , INTENT(in), DIMENSION(jpi,jpj,jpk)           ::   pva    ! mask flux array at V-point
      REAL(wp) , INTENT(in), DIMENSION(jpi,jpj)    , OPTIONAL ::   pmsk   ! Optional 2D basin mask
      !!
      INTEGER                           :: ji, jj, jk ! dummy loop arguments
      REAL(wp), POINTER, DIMENSION(:,:) :: p_fval     ! return function value
#if defined key_mpp_mpi
      INTEGER, DIMENSION(1) ::   ish
      INTEGER, DIMENSION(2) ::   ish2
      INTEGER               ::   ijpjjpk
#endif
#if defined key_mpp_mpi
      REAL(wp), POINTER, DIMENSION(:) ::   zwork    ! mask flux array at V-point
#endif
      !!--------------------------------------------------------------------
      !
#if defined key_mpp_mpi
      ijpjjpk = jpj*jpk
      CALL wrk_alloc( jpj*jpk, zwork )
#endif

      p_fval => p_fval2d

      p_fval(:,:) = 0._wp
      !
      IF( PRESENT( pmsk ) ) THEN 
         DO jk = 1, jpkm1
            DO jj = 2, jpjm1
!!gm here, use of tmask_i  ==> no need of loop over nldi, nlei....
               DO ji =  nldi, nlei   ! No vector optimisation here. Better use a mask ?
                  p_fval(jj,jk) = p_fval(jj,jk) + pva(ji,jj,jk) * e1v(ji,jj) * fse3v(ji,jj,jk) * pmsk(ji,jj)
               END DO
            END DO
         END DO
      ELSE 
         DO jk = 1, jpkm1
            DO jj = 2, jpjm1
               DO ji =  nldi, nlei   ! No vector optimisation here. Better use a mask ?
                  p_fval(jj,jk) = p_fval(jj,jk) + pva(ji,jj,jk) * e1v(ji,jj) * fse3v(ji,jj,jk) * tmask_i(ji,jj)
               END DO
            END DO
         END DO
      END IF
      !
#if defined key_mpp_mpi
      ijpjjpk = jpj*jpk
      ish(1) = ijpjjpk  ;   ish2(1) = jpj   ;   ish2(2) = jpk
      zwork(1:ijpjjpk) = RESHAPE( p_fval, ish )
      CALL mpp_sum( zwork, ijpjjpk, ncomm_znl )
      p_fval(:,:) = RESHAPE( zwork, ish2 )
#endif
      !
#if defined key_mpp_mpi
      CALL wrk_dealloc( jpj*jpk, zwork )
#endif
      !
   END FUNCTION ptr_vjk


   FUNCTION ptr_tjk( pta, pmsk )   RESULT ( p_fval )
      !!----------------------------------------------------------------------
      !!                    ***  ROUTINE ptr_tjk  ***
      !!
      !! ** Purpose :   i-sum computation of e1t*e3t * a tracer field
      !!
      !! ** Method  : - i-sum of mj(pta) using tmask
      !!
      !! ** Action  : - p_fval: i-sum of e1t*e3t*pta
      !!----------------------------------------------------------------------
      !!
      REAL(wp) , INTENT(in), DIMENSION(jpi,jpj,jpk) ::   pta    ! tracer flux array at T-point
      REAL(wp) , INTENT(in), DIMENSION(jpi,jpj)     ::   pmsk   ! Optional 2D basin mask
      !!
      INTEGER                           :: ji, jj, jk   ! dummy loop arguments
      REAL(wp), POINTER, DIMENSION(:,:) :: p_fval       ! return function value
#if defined key_mpp_mpi
      INTEGER, DIMENSION(1) ::   ish
      INTEGER, DIMENSION(2) ::   ish2
      INTEGER               ::   ijpjjpk
#endif
#if defined key_mpp_mpi
      REAL(wp), POINTER, DIMENSION(:) ::   zwork    ! mask flux array at V-point
#endif
      !!-------------------------------------------------------------------- 
      !
#if defined key_mpp_mpi
      ijpjjpk = jpj*jpk
      CALL wrk_alloc( jpj*jpk, zwork )
#endif

      p_fval => p_fval2d

      p_fval(:,:) = 0._wp
      DO jk = 1, jpkm1
         DO jj = 2, jpjm1
            DO ji =  nldi, nlei   ! No vector optimisation here. Better use a mask ?
               p_fval(jj,jk) = p_fval(jj,jk) + pta(ji,jj,jk) * e1t(ji,jj) * fse3t(ji,jj,jk) * pmsk(ji,jj)
            END DO
         END DO
      END DO
#if defined key_mpp_mpi
      ijpjjpk = jpj*jpk
      ish(1) = jpj*jpk   ;   ish2(1) = jpj   ;   ish2(2) = jpk
      zwork(1:ijpjjpk)= RESHAPE( p_fval, ish )
      CALL mpp_sum( zwork, ijpjjpk, ncomm_znl )
      p_fval(:,:)= RESHAPE( zwork, ish2 )
#endif
      !
#if defined key_mpp_mpi
      CALL wrk_dealloc( jpj*jpk, zwork )
#endif
      !    
   END FUNCTION ptr_tjk

   !!======================================================================
END MODULE diaptr_oce
